(function() {
    /********************
    *   ページ判定用定数  *
    *********************/
    var PAGE_DEFINE = {
        /**
         * ---PAGE_PROPの設定について---
         * PAGE_PROPに設定したものを，ページ判定対象とします。
         * 各項目は以下の通りに設定します。
         * TYPE,KEY:（必須）
         * 　ページ判定ための検索条件を配列で指定。一つの場合も[]でくくり，配列とする。
         * 　DOM要素の名前をTYPE，検索のキーワードをKEYとする。TYPEとKEYは，複数条件指定可能。
         * 　ただし，設定時は以下のようにすること。
         * 　・対応するTYPEとKEYはセットで，各配列の同じINDEXに設定すること。
         * 　・条件の数が多いものを上に配置すること
         * 　　例：LOGIN(条件１つ)よりERROR_MESSAGE(条件２つ)のほうが上
         * ID:（必須）
         * 　アプリに返すPageIDに設定する値。
         * ERROR:（任意）
         * 　戻り値に指定するエラータイトル，エラーメッセージを取得するかどうか。取得する場合はtrueを指定。
         * CONTENTS:（任意）
         * 　戻り値contentsに設定する項目を指定。
         * 　ここに指定した値を，CONTENTS_TYPEにも追加すること。
         * 　'json'：JSON形式ページのデータを取得
         * ---------------------------
         */
        PAGE_PROP : {
            /** TS認証エラー */
            TS_ERROR_MESSAGE : {
                        TYPE : ['form','class'],
                        KEY : ['FbTP01020101','error_message'],
                        ID : 'authError',
                        ERROR : true
            },
            /** LEXUS認証エラー */
            LX_ERROR_MESSAGE : {
                        TYPE : ['form','class'],
                        KEY : ['FbLP01020101','error_message'],
                        ID : 'authError',
                        ERROR : true
            },
            /** TS3ログインページ */
            LOGIN_TS3   : {
                            TYPE : ['form'],
                            KEY : ['FbTP01020101'],
                            ID : 'login'
                        },
            /** LEXUSログインページ */
            LOGIN_LEXUS : {
                            TYPE : ['form'],
                            KEY : ['FbLP01020101'],
                            ID : 'login'
                        },
            /** 適用値引き単価ページ */
            FUEL_DISCOUNT : {
                            TYPE : ['form'],
                            KEY : ['FbADME151'],
                            ID : 'fuelDiscount'
                        },
            /** Myページトップ */
            /** ENEOS */
            MYPAGE_TOP_ENEOS  : {
                            TYPE : ['form'],
                            KEY : ['FbADME013'],
                            ID : 'myPageTop'
                        },
            /** TS Cubic */
            MYPAGE_TOP_TSCUBIC  : {
                            TYPE : ['form'],
                            KEY : ['FbADMT014R'],
                            ID : 'myPageTop'
                        },
            /** TS3 家族 */
            MYPAGE_TOP_TS3_K  : {
                            TYPE : ['form'],
                            KEY : ['FbTP01030110'],
                            ID : 'myPageTop'
                        },
            /** CGCカード */
            MYPAGE_TOP_CGC  : {
                            TYPE : ['form'],
                            KEY : ['FbADMC144'],
                            ID : 'myPageTop'
                        },
            /** LEXUS 本会員 */
            MYPAGE_TOP_LEXUS  : {
                            TYPE : ['form'],
                            KEY : ['FbADMQ043'],
                            ID : 'myPageTop'
                        },
            /** LEXUS 家族会員 */
            MYPAGE_TOP_LEXUS_K  : {
                            TYPE : ['form'],
                            KEY : ['FbLP01030116'],
                            ID : 'myPageTop'
                        },
            /** TS3 JSONページ */
            TS3_JSON_PAGE   : {
                            TYPE : ['id'],
                            KEY : ['TP01030111'],
                            ID : 'json',
                            CONTENTS : 'json'
                        },
            /** LEXUS JSONページ */
            LEXUS_JSON_PAGE : {
                            TYPE : ['id'],
                            KEY : ['LP01030117'],
                            ID : 'json',
                            CONTENTS : 'json'
                        },
            /** TS3 カード選択 JSONページ*/
            TS_CHOOSE_CARD_JSON_PAGE : {
                            TYPE : ['form'],
                            KEY : ['FbTP01030112'],
                            ID : 'chooseCardJSON',
                            CONTENTS : 'json'
                        },
            /** LEXUS カード選択 JSONページ*/
            LEXUS_CHOOSE_CARD_JSON_PAGE : {
                            TYPE : ['form'],
                            KEY : ['FbLP01030119'],
                            ID : 'chooseCardJSON',
                            CONTENTS : 'json'
                        },
            /** カード選択ページ */
            CHOOSE_CARD : {
                            TYPE : ['id'],
                            KEY : ['chooseCard'],
                            ID : 'chooseCard'
                        },
            /** アクセス制御エラー */
            ERROR_ACCESS_CONTROL : {
                            TYPE : ['id'],
                            KEY : ['errorAccessControl'],
                            ID : 'accessControlError',
                            ERROR : true
                        },
            /** 画面遷移エラー */
            ERROR_PAGE_TRANSITION : {
                            TYPE : ['id'],
                            KEY : ['errorPageTransition'],
                            ID : 'pageTransitionError',
                            ERROR : true
                        },
            /** システムエラー */
            ERROR_SYSTEM : {
                            TYPE : ['id'],
                            KEY : ['errorSystem'],
                            ID : 'systemError',
                            ERROR : true
                        },
            /** 計画停止 */
            STOP_PLAN   : {
                            TYPE : ['id'],
                            KEY : ['sorry1'],
                            ID : 'sorry1',
                            ERROR : true
                        },
            /** サービス停止(緊急) */
            STOP_EMERGENCY : {
                            TYPE : ['id'],
                            KEY : ['sorry2'],
                            ID : 'sorry2',
                            ERROR : true
                        },
            /** ID強制変更 */
            PAGE_CHANGE_ID  : {
                            TYPE : ['id'],
                            KEY : ['pageIdChange'],
                            ID : 'pageIdChange',
                            ERROR : true
                        },
            /** 退会済会員 */
            UNSUBSCRIVE : {
                            TYPE : ['title'],
                            KEY : ['処理中にエラーが発生しました'],
                            ID : 'unsubscribedMembers',
                            ERROR : true
                        },
            /** 汎用エラー */
            COMMON_ERROR : {
                            TYPE : ['id'],
                            KEY : ['errorVariableDisp'],
                            ID : 'commonError',
                            ERROR : true
                        }
        },
        /** Contentsの種類 */
        /** 取得するContentsを増やす場合は，ここに追加してPAGE_PROPに設定する。*/
        CONTENTS_TYPE : {
            JSONID   : 'json'
        },
        /** その他未定義 */
        UNDEFINED : 'undefined'
    };

    /********************
    *   json作成クラス   *
    *********************/
    /** コンストラクタ */
    function JsonData(){
        this.pageId = PAGE_DEFINE.UNDEFINED;
        this.formName = '';
        this.errorTitle = '';
        this.errorMessage = '';

        /** contentsを生成するkeyとvalueが設定される */
        this.key = undefined;
        this.value = undefined;
    }

    /**
     * ObjectからJsonを作成して返す処理
     * @return jsonに加工した文字列
     */
    JsonData.prototype.getJson = function(){
        var result = {
            pageId : '',
            formName : '',
            errorTitle : '',
            errorMessage : '',
            contents : {}
        };
        result.pageId = this.pageId;
        result.formName = this.formName;
        result.errorTitle = this.errorTitle;
        result.errorMessage = this.errorMessage;
        /** keyとvalueが設定されているときは，resultに追加する */
        if( this.key !== undefined && this.value !== undefined ) {
            result.contents[this.key] = this.value;
        }
        return JSON.stringify(result);
    };

    /********************
    * スクレイピングクラス *
    *********************/
    /** コンストラクタ */
    function Scraping() {
        /** PAGE_DEFINE.PAGE_PROPのキーを設定。 */
        /** ここで設定されたキーに対応するデータをスクレイピングにより取得する。 */
        this.pageKey = this.judgePage();
   }


    /**
     * pageの種類を判定する。
     * 定義済みのpageの場合は，キー情報を返す。未定義の場合は"undefined"
     * @return pageKey PAGE_DEFINE.PAGE_PROPのキー情報
     */
    Scraping.prototype.judgePage = function() {
        var pageKey = undefined;
        /** PAGE_PROPの項目ごとにチェック */
        $.each( PAGE_DEFINE.PAGE_PROP, function( key , value ) {
            var types = value['TYPE'];
            var keys = value['KEY'];
            var numOfMatched = 0;
            /** TYPEに設定された項目をKEYの値でチェック，マッチした数をカウント */
            for( var i=0; i<types.length; i++ ) {
                /** formによるページ判定 */
                if( types[i] === 'form' ) {
                    if( $('form[name=' + keys[i] + ']').length !== 0  ) {
                        numOfMatched++;
                    }
                }
                /** idによるページ判定 */
                else if( types[i] === 'id' ) {
                    if( $('#' + keys[i]).length !== 0 ) {
                        numOfMatched++;
                    }
                }
                /** classによるページ判定 */
                else if ( types[i] === 'class' ) {
                    if( $('.' + keys[i]).length !== 0 ) {
                        numOfMatched++;
                    }
                }
                /** titleによるページ判定 */
                else if( types[i] === 'title' ) {
                    if( $('title').text().indexOf(keys[i]) > -1 ) {
                        numOfMatched++;
                    }
                }
            }
            if( types.length === numOfMatched ) {
                /** チェックした数とマッチした数が一致したらOK */
                pageKey = key;
                return false;
            }
        });
        return pageKey;
    };

    /**
     * pageIdを取得する
     * @return pageId
     */
    Scraping.prototype.getPageId = function(){
        if( PAGE_DEFINE.PAGE_PROP[this.pageKey] ) {
            return PAGE_DEFINE.PAGE_PROP[this.pageKey]['ID'];
        }
        return PAGE_DEFINE.UNDEFINED;
    };

    /**
     * formNameを取得する。
     * @return formName 不要な場合は ""
     */
    Scraping.prototype.getFormName = function() {
        var $form = $('form');
        if( $form[0] ) {
           return $form[0].name;
        }
        return '';
    };

    /**
     * Error関連の情報を取得する。
     * ERRORフラグが立っているページのみ。
     * ページごとに情報取得方法が違う。
     * @return {*} TitleとMessageをキーとしたデータのオブジェクト。
     */
    Scraping.prototype.getErrorTitleAndMessage = function() {
        var result = {
            'title' : '',
            'message' : '',
        };
        if( PAGE_DEFINE.PAGE_PROP[this.pageKey]['ERROR'] ) {
            switch(this.pageKey) {
                case 'TS_ERROR_MESSAGE':
                case 'LX_ERROR_MESSAGE':
                    /** title */
                    /** なし */
                    /** message */
                    var message = '';
                    $('.error_message').find('li').each(function() {
                        message += $(this).text() + '\n';
                    });
                    result['message'] = message;
                    break;
                case 'ERROR_ACCESS_CPNTROL':
                    /** title */
                    /** なし */
                    /** message */
                    /** なし */
                    break;
                case 'ERROR_PAGE_TRANSITION':
                    /** title */
                    /** なし */
                    /** message */
                    /** なし */
                    break;
                case 'ERROR_SYSTEM':
                    /** title */
                    /** なし */
                    /** message */
                    if( $('#Main .content p').length > 0 ) {
                        var message = '';
                        $.each($('#Main .content p'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;
                    } else if( $('#dContents .dSectionNormal p').length > 0 ) {
                        var message = '';
                        $.each($('#dContents .dSectionNormal p'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;
                    }
                    break;
                case 'STOP_PLAN':
                    /** title */
                    if( $('#contentsflat h2').length > 0 ) {
                        result['title'] = $('#contentsflat h2')[0].innerText;
                    } else if ( $('#dContents h2').length > 0 ) {
                    	result['title'] = $('#dContents h2')[0].innerText;
                    }
                    /** message */
                    if( $('#contentsflat p').length > 0 ) {
                        var message = '';
                        $.each($('#contentsflat p'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;
                    } else if( $('#dContents p.pLast').length > 0 ) {
                        var message = '';
                        $.each($('#dContents p.pLast'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;                    	
                    }
                    break;
                case 'STOP_EMERGENCY':
                    /** title */
                    if( $('h1').length > 0 ) {
                        result['title'] = $('h1')[0].innerText;
                    }
                    /** message */
                    if( $('p').length > 0 ) {
                        var message = '';
                        $.each($('p'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;
                    }
                    break;
                case 'PAGE_CHANGE_ID':
                    /** title */
                    /** なし */
                    /** message */
                    /** なし */
                    break;
                case 'UNSUBSCRIVE':
                    /** title */
                    if( $('#Main h2').length > 0 ) {
                        result['title'] = $('#Main h2')[0].innerText;
                    }
                    /** message */
                    if( $('#Main p').length > 0 ) {
                        var message = '';
                        $.each($('#Main p'),function(index, elem) {
                            message += elem.innerText + '\n';
                        });
                        result['message'] = message;
                    }
                    break;
                case 'COMMON_ERROR':
                    /** message */
                    if( $('.errorText').length > 0 ) {
                        var message = $('.errorText')[0].innerText;
                        result['message'] = message;
                    }
                    break;
                default:
                    break;
            }
        }
        return result;
    };

    /**
     * Contents情報を取得する。
     * CONTENTSに値が設定されている場合は，値に対応するデータを取得。
     * contents取得対象でない場合は，undefinedが返る。
     * @return key/value keyをCONTENTSの値，valueをそのデータとする。
     */
    Scraping.prototype.getContents = function() {
        var result = undefined;
        var page_prop = PAGE_DEFINE.PAGE_PROP[this.pageKey];
        if( page_prop['CONTENTS'] !== undefined ) {
            var contents = page_prop['CONTENTS'];
            result = {};
            if( contents === PAGE_DEFINE.CONTENTS_TYPE.JSONID ) {
                result['key'] = contents;
                result['value'] = this.getJsonData($('#json-data')[0].text);
            }
        }
        return result;
    };

    /**
     * jsonコンテンツからJSONデータを取得する
     * @param {*} data jsonを含む文字列
     */
    Scraping.prototype.getJsonData = function(data) {
        var DELIMITER = '{';
        if( /DELIMITER/.test(data) > -1 ) {
            var result = data.slice(data.indexOf(DELIMITER));
            /** JSON形式に変換 */
            result = JSON.parse(result);

            return result;
        }
        else {
            return undefined;
        }
    };
    /********************
    *      実行処理      *
    *********************/
    /** json作成用モデルを生成 */
    var jsonData = new JsonData();
    /** スクレイピングクラスを生成 */
    var scraping = new Scraping();

    /** まずはページIDを取得して，既知のページかどうか確認 */
    jsonData.pageId = scraping.getPageId();
    if( jsonData.pageId !== PAGE_DEFINE.UNDEFINED ) {
        /** 既知のページだったら他の値を設定 */
        jsonData.formName = scraping.getFormName();
        var errorData = scraping.getErrorTitleAndMessage();
        jsonData.errorTitle = errorData['title'];
        jsonData.errorMessage = errorData['message'];
        var contents = scraping.getContents();
        if( contents !== undefined ) {
            jsonData.key = contents['key'];
            jsonData.value = contents['value'];
        }
    }

    /** jsonを返す処理 */
    return jsonData.getJson();
}());
